// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.annotatedlineeditor.handler;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor;

/**
 * <p>NewObjectHandler class.</p>
 *
 * @author p.ciardelli
 * @created 14.08.2009
 * @version 1.0
 */
public class NewObjectHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		IEditorInput input = editor.getEditorInput();
		
		if (editor instanceof AnnotatedLineEditor) {
			
			Object key = ((Event)event.getTrigger()).data;
			if (key != null) {
				if(!(key instanceof SpecimenOrObservationType)) {
					String text = ((Event)event.getTrigger()).text; 
					
					//FIXME : This should probably go into some ValidatorFactory
					IInputValidator nonEmptyInputValidator = null;
					//FIXME : This is a workaround to not allow empty strings in the 
					//        input dialog for User and Group entities. 
					//        Normally this should be default
					//        behaviour, so we need to discuss whether this handler
					//        should be used to handle the creating new entities of
					//        type other than User and Group.
					//        Once #4348 is fixed this check can be removed.
					if(text.equals("User") || text.equals("Group")) {
						nonEmptyInputValidator = new IInputValidator() {
							public String isValid(String text) {							
								if(text == null || text.isEmpty()) {
									return "Input cannot be empty";
								}
								return null;
							}			
						};
					}
					InputDialog dialog = new InputDialog(HandlerUtil.getActiveShell(event), 
							"Create " + text, 
							"Enter new " + text, "",
							nonEmptyInputValidator); 

					if (dialog.open() != Window.CANCEL) { 
						((AnnotatedLineEditor) editor).createAnnotatedLineNewObject(key, dialog.getValue()); 
					}
				} else {
					((AnnotatedLineEditor) editor).createAnnotatedLineNewObject(key, "Untitled"); 
				}
				
			} else {
				((AnnotatedLineEditor) editor).createAnnotatedLineNewObject();
			}
		}
		return null;
	}
}
