// $Id$
/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.handler;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.texteditor.IDocumentProvider;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotation;
import eu.etaxonomy.taxeditor.annotatedlineeditor.LineAnnotationModel;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 12.05.2015
 *
 */
public class ConvertPerson2TeamHandler extends AbstractHandler {

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		ISelection selection = HandlerUtil.getCurrentSelection(event);

		IEditorPart editor = HandlerUtil.getActiveEditor(event);

		IEditorInput input = editor.getEditorInput();
		if (editor.isDirty()){
			boolean proceed = MessageDialog.openQuestion(null,
					"Save changes", "You have made changes that must be saved before this query can be executed. Would you like to proceed?");
			if (proceed) {
				editor.doSave(null);
			} else {
				return null;
			}
		}
		if((input instanceof IEntityPersistenceService) && (selection instanceof IStructuredSelection)){


			IDocumentProvider provider = ((BulkEditor) editor).getDocumentProvider();
			LineAnnotationModel model =
					(LineAnnotationModel) provider.getAnnotationModel(input);


			IStructuredSelection structuredSelection = (IStructuredSelection) selection;

			IEntityPersistenceService persistenceService = (IEntityPersistenceService) input;

			Team team ;
			for(Object object : structuredSelection.toList()){
				LineAnnotation annotation = (LineAnnotation) model.getAnnotation(object);
				team = null;
				if (object instanceof Person){
					Person person = HibernateProxyHelper.deproxy(object, Person.class);
					try {
						team = CdmStore.getService(IAgentService.class).convertPerson2Team(person.getUuid());
					} catch (IllegalArgumentException e) {
						MessagingUtils.errorDialog("Can not convert Person into a Team", null, e.getLocalizedMessage(), TaxeditorBulkeditorPlugin.PLUGIN_ID,e, true);
					} catch (MergeException e) {
						MessagingUtils.informationDialog("Convert not possible", "Person can not be transformed into team as it is already part of a team.");
					}
				}else{
					MessagingUtils.informationDialog("Can not convert Person into a Team", "convert Person to Team can only be called on a person.");
				}

				if (team != null){
					((BulkEditor) editor).removeAnnotatedLine(annotation);
					((BulkEditor) editor).createAnnotatedLine(team);
				}
			}
		}
		return null;
	}

}
