// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.List;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.strategy.merge.DefaultMergeStrategy;
import eu.etaxonomy.cdm.strategy.merge.IMergable;
import eu.etaxonomy.cdm.strategy.merge.IMergeStrategy;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.cdm.strategy.merge.MergeMode;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.AgentCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.IdentifiableEntitySortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.internal.TaxeditorBulkeditorPlugin;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AgentEditorInput extends AbstractBulkEditorInput<TeamOrPersonBase> {

	private static final long serialVersionUID = 3387950621617078479L;

	public static final String ID = "bulkeditor.input.author";

	private static AgentEditorInput instance;

	public static String getID() {
		return ID;
	}


	@Override
    public String getName() {
		return BulkEditorInputType.AGENT.label;
	}


	@Override
    public String getToolTipText() {
		return getName();
	}

	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new AgentEditorInput();
		}
		return instance;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isMergingEnabled() {
		return true;
	}


	/** {@inheritDoc} */
	@Override
	public boolean isConvertingEnabled() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
    public boolean merge(TeamOrPersonBase entity, TeamOrPersonBase mergeTarget) {
		if (entity instanceof IMergable) {
			try {
				if(entity instanceof Person && mergeTarget instanceof Person) {
				    IMergeStrategy strategy = DefaultMergeStrategy.NewInstance(Person.class);
				    strategy.setMergeMode("institutionalMemberships", MergeMode.FIRST);
				    CdmStore.getCommonService().merge(mergeTarget.getUuid(), entity.getUuid(), Person.class);
				} else {
				    IMergeStrategy strategy = DefaultMergeStrategy.NewInstance(TeamOrPersonBase.class);
				    CdmStore.getCommonService().merge(mergeTarget.getUuid(), entity.getUuid(), TeamOrPersonBase.class);
				}
			} catch (MergeException e) {
				MessagingUtils.errorDialog("Bulk Editor Merge Error",
						this,
						"Could not merge chosen objects of type " + entity.getClass().getName(),
						TaxeditorBulkeditorPlugin.PLUGIN_ID,
						e,
						true);
			}
		}
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public List<TeamOrPersonBase> listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.getSearchManager().findTeamOrPersons(configurator);
	}

	/** {@inheritDoc}
	@Override
	public AgentBase loadEntity(UUID uuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(IAgentService.class).load(uuid, propertyPaths);
	}*/


	public boolean delete(AgentBase entity) throws ReferencedObjectUndeletableException  {
			return CdmStore.getService(IAgentService.class).delete(entity) != null;

	}

	/** {@inheritDoc} */
	public boolean save(AgentBase entity) {

		return CdmStore.getService(IAgentService.class).saveOrUpdate(entity) != null;
	}

	@Override
	public List<IBulkEditorSortProvider<TeamOrPersonBase>> getSortProviders() {
		List<IBulkEditorSortProvider<TeamOrPersonBase>> sortProviders = super.getSortProviders();

		sortProviders.add(0, new IdentifiableEntitySortProvider<TeamOrPersonBase>());

		return sortProviders;
	}


	@Override
	protected IEntityCreator<TeamOrPersonBase> createEntityCreator() {
		return new AgentCreator();
	}

	@Override
	public TeamOrPersonBase save(TeamOrPersonBase entity) {
	    if(CdmStore.getCurrentSessionManager().isRemoting()) {
	        return (TeamOrPersonBase) CdmStore.getService(IAgentService.class).merge(entity, true).getMergedEntity();
	    } else {
	        CdmStore.getService(IAgentService.class).saveOrUpdate(entity) ;
	        return entity;
	    }
	}

	@Override
	public boolean delete(TeamOrPersonBase entity)
			throws ReferencedObjectUndeletableException {
		// TODO Auto-generated method stub
		return false;
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {

    }


}
