package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.FindTaxaAndNamesConfiguratorImpl;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.input.entitycreator.TaxonCreator;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class TaxonEditorInput extends AbstractBulkEditorInput<TaxonBase>{

	private static TaxonEditorInput instance;
	/** Constant <code>ID="bulkeditor.input.taxon"</code> */
	public static final String ID = "bulkeditor.input.taxon";



	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new TaxonEditorInput();
		}

		return instance;
	}


	@Override
	public String getName() {
		return BulkEditorInputType.TAXON.label;
	}

	@Override
	public String getToolTipText() {
		return getName();
	}


	/**
	 * <p>getID</p>
	 *
	 * @return a {@link java.lang.Object} object.
	 */
	public static Object getID() {
		return ID;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean isMergingEnabled() {
		return false;
	}
	@Override
	public TaxonBase save(TaxonBase entity) {
	    if(CdmStore.getCurrentSessionManager().isRemoting()) {
	        return CdmStore.getService(ITaxonService.class).merge(entity, true).getMergedEntity();
	    } else {
	        CdmStore.getService(ITaxonService.class).saveOrUpdate(entity);
	        return entity;
	    }
	}



	/** {@inheritDoc} */
	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		IFindTaxaAndNamesConfigurator<TaxonBase> newConfig = new FindTaxaAndNamesConfiguratorImpl<TaxonBase>();
		newConfig.setTitleSearchString(configurator.getTitleSearchStringSqlized());
		newConfig.setMatchMode(MatchMode.ANYWHERE);
		List<TaxonBase> taxa =  CdmStore.getSearchManager().findTaxa(configurator);
		List<TaxonBase> taxaCopy = new ArrayList<TaxonBase>();
		for (TaxonBase taxon:taxa){

			if (taxon instanceof Taxon){
				taxaCopy.add(HibernateProxyHelper.deproxy(taxon, Taxon.class));
			}else{
				taxaCopy.add(HibernateProxyHelper.deproxy(taxon, Synonym.class));
			}
		}
		return taxaCopy;
	}


	@Override
	protected TaxonBase loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{});
		return CdmStore.getService(ITaxonService.class).load(entityUuid, propertyPaths);
	}



	/** {@inheritDoc}
	 * @throws ReferencedObjectUndeletableException */
	@Override
    public boolean delete(TaxonBase entity) {
		return CdmStore.getService(ITaxonService.class).delete(entity) != null;
	}
	@Override
	protected IEntityCreator<TaxonBase> createEntityCreator() {
		return new TaxonCreator();
	}


    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.editor.CdmEntitySessionInput#merge()
     */
    @Override
    public void merge() {

    }

}
